###############################################################
######### Statistical Learning: Intoduction to R      #########
######### Rosario Barone                              #########
######### email: rosario.barone@uniroma2.it           #########
###############################################################


#################################################################
#################################################################
#################################################################

f<-function(x, a, b){
  y<-a+b*x^2
  return(y)
}

curve(f(x=x,a=3,b=3), xlim=c(-10,10))

f(x=0,a=1,b=1)
f(x=1,a=1,b=1)
f(x=-1,a=1,b=1)


##### Upload data #######

setwd("~/Desktop/Lectures/Statistical Learning/R practice")

# Here we deal with continuous quantitative variables (all real numbers)

data <- read.csv("income.txt", header=T, dec=".",sep="")
mode(data)
income<-data$income
mode(income)
dim(income)
n<-length(income)
#Check if there are missing values
length(which(is.na(income)))
#Check if there are zeros
length(which(income==0))

### ASSUMPTION 1:
# Zeros are NA (unobserved values)
income[income==0]=NA

### ASSUMPTION 2:
length(which(is.na(income)))
# Since 14 values are MISSING AT RANDOM, we remove the elements:
income<-na.omit(income)
n<-length(income)

# We import the income data of some individuals
# then, we visualize the empirical distribution graphically
# using the frequency histogram
hist(income, xlab="Net disposable income",
     main="Empirical distribution of the income", freq=TRUE)

# or, by using the histogram of the density
hist(income, xlab="Net disposable income",
     main="Empirical distribution of the income", freq=FALSE)

# When "freq = F", we have the density, which means that the area of the histogram adds to 1
# When "freq = T", we have the frequencies, i.e. the area of the histogram adds to N (i.e., the
# number of observations).

#We re-scale the income
scaled.income=income/1000
# so we have the same histogram with a more readable scale:
hist(scaled.income, xlab="Net disposable income (1000$=1K$)",
     main="Empirical distribution of the income", freq=F)

# so we have the same histogram with a more readable scale:
hist(scaled.income, xlab="Net disposable income (1000$=1K$)",
     main="Empirical distribution of the income", freq=F)


# It can be seen that there is a strong positive skewness, in fact most of the data
# lies on the left side of the histogram (corresponding to the income of the most
# low), while there are few high incomes (those on the right side of the histogram).

#### Maximum Likelihood Inference ###

# 1) Assume a statistical model
# 2) Calculate de likelihood function
# Two (equivalent) approaches:
# 1) Derive analitically the ML estimator(s) for the model parameter(s)
# 2) Approximate numerically the likelihood function 


################################################################################ 
################################################################################ 
################################################################################ 
##############               EXPONENTIAL MODEL                    ##############
################################################################################ 
################################################################################ 
################################################################################ 

# Y~Exp(lambda)

#### GENERATE VALUES OF A RANDOM VARIABLE
X<-scaled.income
n<-length(X)
lambda_ML<-1/(sum(X)/n)

## Calculating standard errors:
## theta_ML  ~ N(theta, 1/I(theta))
## where I(theta) is the Fisher Information
## I(theta)=E(-d^2 L''(theta)/dtheta^2 )

se.lambda_ML<-(lambda_ML^2)/n


#### Alternatively we can maximize the likelihood numerically

exponential.lik<-function(lambda,x){
  n<-length(x)
  logl<-n*log(lambda)-lambda*sum(x) 
  return(logl) }

#Check: we can evaluate the function in infinite points (lambda>0)
exponential.lik(lambda = 0.1,x=X)
exponential.lik(lambda = 0.6,x=X)
exponential.lik(lambda = 0.7,x=X)
exponential.lik(lambda = 0.9,x=X)

exponential.lik.plot1<-function(lambda){
  x<-X
  n<-length(x)
  logl<-n*log(lambda)-lambda*sum(x) 
  return(logl) }
plot(exponential.lik.plot1,ylab="l(lambda)", xlab="lambda")

#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 
#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 
#### #### #### #### #### #### COMPUTATIONAL ISSUES #### #### #### #### #### ###
#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 
#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 

# The optimizer MINIMIZES the functions wrt the parameters of interest. Hence:

exponential.lik<-function(lambda,x){
  n<-length(x)
  logl<-n*log(lambda)-lambda*sum(x) 
  return(-logl) }

exponential.lik.plot2<-function(lambda){
  x<-X
  n<-length(x)
  logl<-n*log(lambda)-lambda*sum(x) 
  return(-logl) }
plot(exponential.lik.plot2, ylab = "-l(lambda)", xlab="lambda")

#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 
#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 
#### #### #### #### #### #### #### #### #### #### #### #### #### #### #### #### 

ML_numeric<-optim(par=0.1,fn=exponential.lik,x=X,hessian=TRUE,method="BFGS")
lambda_ML_numeric<-ML_numeric$par
se.lambda.numeric<-diag(solve(ML_numeric$hessian))

#The MLE estimator is asymptotically Noramal:
confidence.interval.upper<-c(lambda_ML_numeric-qnorm(0.975)*se.lambda.numeric,lambda_ML+qnorm(0.975)*se.lambda_ML)
confidence.interval.lower<-c(lambda_ML-qnorm(0.975)*se.lambda_ML,lambda_ML_numeric+qnorm(0.975)*se.lambda.numeric)

lambda.estimated<-c(lambda_ML,lambda_ML_numeric)
se.lmabda.estimated<-c(se.lambda_ML,se.lambda.numeric)

results<-cbind(lambda.estimated,se.lmabda.estimated,confidence.interval.lower,confidence.interval.upper)
colnames(results)<-c("Estimate", "Standard Error", "5%", "95%")
rownames(results)<-c("Analitical", "Numerical")
results

hist(X,xlab = "Income (thousands of dollars)",ylab = "Density", freq = TRUE, main="Exponential Model fitting")
curve(dexp(x, rate = lambda.estimated), add=TRUE, col=2, lwd=2)

################################################################################ 
################################################################################ 
################################################################################ 
##############                NORMAL MODEL                        ##############
################################################################################ 
################################################################################ 
################################################################################

X<-log(scaled.income)
length(X)
X[X==-Inf]=NA
X<-na.omit(X)
n<-length(X)
mu_ML<-(1/n)*sum(X)
sigma2_ML<-(1/n)*sum((X-mu_ML)^2) 

se.mu<-1/(n/sigma2_ML)
se.sigma2<-1/(n/(2*sigma2_ML^2))


normal.lik<-function(theta,x){
 mu<-theta[1]
 sigma2<-theta[2]
 n<-length(x)
 llik= -n/2*log(sigma2)-1/(2*sigma2)*sum((x-mu)^2)
 return(-llik)
}

# Theta is the vector c(mu,sigma2)
theta_ML<-optim(par=c(1,1),fn=normal.lik,x=X,hessian=T,method="BFGS")
OI<-solve(theta_ML$hessian)
se<-diag(OI)

hist(X,xlab = "log Income (thousands of dollars)",ylab = "Density", freq = FALSE, main="Normal model fitting")
curve(dnorm(x, mean = mu_ML, sd=sqrt(sigma2_ML)), add=TRUE, col=2, lwd=2)

################################################################################ 
################################################################################ 
################################################################################ 
##############               WEIBULL MODEL                        ##############
################################################################################ 
################################################################################ 
################################################################################

X<-scaled.income

weibull.loglik<-function(theta,x){
 beta<-theta[1]
 alpha<-theta[2]
 n<-length(x)
 llik<-n*log(beta)+(beta-1)*sum(log(x))-n*beta*log(alpha)-(sum(x^beta)/alpha^beta)
 return(-llik)
}
theta_ML<-optim(par=c(0.1,1),fn=weibull.loglik,x=X,hessian=T,method="BFGS")
theta_ML$par
OI<-solve(theta_ML$hessian)
se<-diag(OI)
hist(X,xlab = "Income (thousands of dollars)",ylab = "Density", freq = FALSE, ylim=c(0,0.02), main="Weibull model fitting")
curve(dweibull(x, shape = theta_ML$par[1], scale=theta_ML$par[2]), add=TRUE, col=2, lwd=2)


################################################################################ 
################################################################################ 
################################################################################ 
##############                 GAMMA MODEL                        ##############
################################################################################ 
################################################################################ 
################################################################################


gamma.loglik<-function(theta,x){
   a<-theta[1]
   b<-theta[2]
   n<-length(x)
  llik<- -n*log(gamma(a))-n*a*log(b)+(a-1)*sum(log(x))-sum(x)/b
  #Computational Issue
  if(a<0|b<0) llik<-NaN
 return(-llik)
}

theta_ML<-optim(par=c(1,1),fn=gamma.loglik,x=X,hessian=T,method="BFGS")
a_ML<-theta_ML$par[1]
b_ML<-theta_ML$par[2]
OI<-solve(theta_ML$hessian)
se<-diag(OI)

hist(X,xlab = "Income (thousands of dollars)",ylab = "Density", freq = FALSE, ylim=c(0,0.025), main="Gamma model fitting")
curve(dgamma(x, shape = a_ML, scale=b_ML), add=TRUE, col=2, lwd=2)



###############################################################################
###############################################################################
###############################################################################
###############################################################################
################# GENERATION OF  VALUES FROM A RANDOM VARIABLE ################
###############################################################################
###############################################################################
###############################################################################
###############################################################################

#Generating data from an Exponential Model
n=100
X<-rexp(n, rate=0.6)

#Estimating the parameter lambda
ML_estimator<-1/(sum(X)/n)
se.lambda<-(ML_estimator^2)/n

### increasing the sample size....
ML_estimator<-NULL
standard.error<-NULL
n.size<-c(10,50,100,1000)
for(n in n.size){
  X<-rexp(n, rate=0.6)
  mle<-1/(sum(X)/n)
  se<-(mle^2)/n
  ML_estimator<-c(ML_estimator,mle)
  standard.error<-c(standard.error,se)
}
results<-rbind(ML_estimator,standard.error)
colnames(results)<-c("n=10","n=100","n=1000","n=10000")
results
#Bias reduces as the sample size increases
#se reduces as the sample size increases

### Maximum Likelihood Estimation (numerically even if it is analitically available) ####
# Generate a Dataset from a Poisson Distribution

X<-rpois(100, lambda=1.6)
poisson.lik<-function(lambda,x){
  n<-length(x)
  logl<-sum(x)*log(lambda)-n*lambda 
  return(-logl) }

#Analogoulsy, the MLE estimator of the poisson distribution: 
n<-length(X)
lambda.pois.ml<-sum(X)/n
se.lambda<-lambda.pois.ml/n

lambda_optim<-optim(par=1,fn=poisson.lik,x=X,hessian=TRUE,method="BFGS")
diag(solve(lambda_optim$hessian))
